import { Scope } from '@tinyslice/core';
export * from '@tinyslice/core';
import * as i0 from '@angular/core';
import { InjectionToken, ApplicationRef, NgModule, Inject, Optional } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subscription } from 'rxjs';

const TINYSLICE_EFFECT_SERVICES = new InjectionToken('TINYSLICE_EFFECT_SERVICES');
const TINYSLICE_ROOT_MODULE_INDICATOR_TOKEN = new InjectionToken('TINYSLICE_ROOT_MODULE_INDICATOR_TOKEN');

const ROOT_STORE = new InjectionToken('ROOT_STORE');

class TinySliceAngularPlugin {
    constructor(application) {
        this.application = application;
        this.sink = new Subscription();
        this.register = (hooks) => {
            this.hooks = hooks;
            this.initialState = JSON.stringify(hooks.initialState);
        };
        this.onError = (error) => {
            console.log('Error from store:', error);
        };
        this.start = () => {
            this.sink.add(this.hooks.state$.subscribe(() => {
                this.application.tick();
            }));
        };
        this.stop = () => {
            this.sink.unsubscribe();
        };
    }
}
class TinySliceModule {
    constructor(rootModuleIndicators, _effectServices) {
        this.rootModuleIndicators = rootModuleIndicators;
        this._effectServices = _effectServices;
        if (this.rootModuleIndicators.length > 1) {
            throw new Error('More than 1 TinySlice root modules were created!');
        }
    }
    static forRoot(initialState, effectServices, facade, storeOptions) {
        return {
            ngModule: TinySliceModule,
            providers: [
                {
                    provide: TINYSLICE_ROOT_MODULE_INDICATOR_TOKEN,
                    useValue: true,
                    multi: true,
                },
                {
                    provide: Scope,
                    useFactory: () => new Scope(),
                },
                {
                    provide: ROOT_STORE,
                    useFactory: (scope, application) => {
                        var _a;
                        (_a = storeOptions === null || storeOptions === void 0 ? void 0 : storeOptions.plugins) === null || _a === void 0 ? void 0 : _a.forEach((plugin) => { var _a; return (_a = plugin.registerAdditionalTrigger) === null || _a === void 0 ? void 0 : _a.call(plugin, () => application.tick()); });
                        return scope.createRootSlice(initialState, storeOptions);
                    },
                    deps: [Scope, ApplicationRef],
                },
                {
                    provide: facade,
                    useClass: facade,
                    deps: [ROOT_STORE, Scope],
                },
                ...effectServices.map((effectService) => ({
                    provide: TINYSLICE_EFFECT_SERVICES,
                    useClass: effectService,
                    multi: true,
                })),
            ],
        };
    }
    /**
     * TODO: deal with nested features!
     */
    static forFeature(key, initialState, reducers = [], effectServices, facade) {
        const featureToken = new InjectionToken(key);
        return {
            ngModule: TinySliceModule,
            providers: [
                {
                    provide: featureToken,
                    useFactory: (rootStore) => rootStore.addSlice(key, initialState, { reducers }),
                    deps: [ROOT_STORE],
                },
                {
                    provide: facade,
                    useClass: facade,
                    deps: [featureToken, Scope],
                },
                ...effectServices.map((effectService) => ({
                    provide: TINYSLICE_EFFECT_SERVICES,
                    useClass: effectService,
                    multi: true,
                })),
            ],
        };
    }
}
TinySliceModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: TinySliceModule, deps: [{ token: TINYSLICE_ROOT_MODULE_INDICATOR_TOKEN }, { token: TINYSLICE_EFFECT_SERVICES, optional: true }], target: i0.ɵɵFactoryTarget.NgModule });
TinySliceModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.7", ngImport: i0, type: TinySliceModule, imports: [CommonModule] });
TinySliceModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: TinySliceModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.7", ngImport: i0, type: TinySliceModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                }]
        }], ctorParameters: function () {
        return [{ type: undefined, decorators: [{
                        type: Inject,
                        args: [TINYSLICE_ROOT_MODULE_INDICATOR_TOKEN]
                    }] }, { type: undefined, decorators: [{
                        type: Inject,
                        args: [TINYSLICE_EFFECT_SERVICES]
                    }, {
                        type: Optional
                    }] }];
    } });

/**
 * Generated bundle index. Do not edit.
 */

export { ROOT_STORE, TINYSLICE_EFFECT_SERVICES, TINYSLICE_ROOT_MODULE_INDICATOR_TOKEN, TinySliceAngularPlugin, TinySliceModule };
//# sourceMappingURL=tinyslice-ngx.mjs.map
